==> strawberry_1.0.23.orig.tar.xz: strawberry-1.0.23/src/playlistparsers/playlistparser.cpp <==
/*
 * Strawberry Music Player
 * This file was part of Clementine.
 * Copyright 2010, David Sansome <me@davidsansome.com>
 *
 * Strawberry is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Strawberry is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Strawberry.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <algorithm>

#include <QObject>
#include <QIODevice>
#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QByteArray>
#include <QString>
#include <QStringList>

#include "core/logging.h"
#include "core/shared_ptr.h"
#include "settings/playlistsettingspage.h"
#include "playlistparser.h"
#include "parserbase.h"
#include "asxiniparser.h"
#include "asxparser.h"
#include "cueparser.h"
#include "m3uparser.h"
#include "plsparser.h"
#include "wplparser.h"
#include "xspfparser.h"

const int PlaylistParser::kMagicSize = 512;

PlaylistParser::PlaylistParser(SharedPtr<CollectionBackendInterface> collection_backend, QObject *parent) : QObject(parent) {

  default_parser_ = new XSPFParser(collection_backend, this);
  parsers_ << default_parser_;
  parsers_ << new M3UParser(collection_backend, this);
  parsers_ << new PLSParser(collection_backend, this);
  parsers_ << new ASXParser(collection_backend, this);
  parsers_ << new AsxIniParser(collection_backend, this);
  parsers_ << new CueParser(collection_backend, this);
  parsers_ << new WplParser(collection_backend, this);

}

QStringList PlaylistParser::file_extensions(const Type type) const {

  QStringList ret;

  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser)) {
      ret << parser->file_extensions();
    }
  }

  std::stable_sort(ret.begin(), ret.end());
  return ret;

}

QStringList PlaylistParser::mime_types(const Type type) const {

  QStringList ret;

  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser) && !parser->mime_type().isEmpty()) {
      ret << parser->mime_type();
    }
  }

  std::stable_sort(ret.begin(), ret.end());

  return ret;

}

QString PlaylistParser::filters(const Type type) const {

  QStringList filters;
  filters.reserve(parsers_.count() + 1);
  QStringList all_extensions;
  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser)) {
      filters << FilterForParser(parser, &all_extensions);
    }
  }

  if (type == Type::Load) {
    filters.prepend(tr("All playlists (%1)").arg(all_extensions.join(" ")));
  }

  return filters.join(";;");

}

QString PlaylistParser::FilterForParser(const ParserBase *parser, QStringList *all_extensions) {

  const QStringList file_extensions = parser->file_extensions();
  QStringList extensions;
  extensions.reserve(file_extensions.count());
  for (const QString &extension : file_extensions) {
    extensions << "*." + extension;
  }

  if (all_extensions) *all_extensions << extensions;

  return tr("%1 playlists (%2)").arg(parser->name(), extensions.join(" "));

}

QString PlaylistParser::default_extension() const {
  QStringList file_extensions = default_parser_->file_extensions();
  return file_extensions[0];
}

QString PlaylistParser::default_filter() const {
  return FilterForParser(default_parser_);
}

ParserBase *PlaylistParser::ParserForExtension(const Type type, const QString &suffix) const {

  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser) && parser->file_extensions().contains(suffix, Qt::CaseInsensitive)) {
      return parser;
    }
  }
  return nullptr;

}

ParserBase *PlaylistParser::ParserForMimeType(const Type type, const QString &mime_type) const {

  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser) && !parser->mime_type().isEmpty() && QString::compare(parser->mime_type(), mime_type, Qt::CaseInsensitive) == 0) {
      return parser;
    }
  }
  return nullptr;

}

ParserBase *PlaylistParser::ParserForMagic(const QByteArray &data, const QString &mime_type) const {

  for (ParserBase *parser : parsers_) {
    if ((!mime_type.isEmpty() && mime_type == parser->mime_type()) || parser->TryMagic(data)) {
      return parser;
    }
  }
  return nullptr;

}

SongList PlaylistParser::LoadFromFile(const QString &filename) const {

  QFileInfo fileinfo(filename);

  // Find a parser that supports this file extension
  ParserBase *parser = ParserForExtension(Type::Load, fileinfo.suffix());
  if (!parser) {
    qLog(Warning) << "Unknown filetype:" << filename;
    return SongList();
  }

  // Open the file
  QFile file(filename);
  if (!file.open(QIODevice::ReadOnly)) return SongList();

  SongList ret = parser->Load(&file, filename, fileinfo.absolutePath());
  file.close();

  return ret;

}

SongList PlaylistParser::LoadFromDevice(QIODevice *device, const QString &path_hint, const QDir &dir_hint) const {

  // Find a parser that supports this data
  ParserBase *parser = ParserForMagic(device->peek(kMagicSize));
  if (!parser) {
    return SongList();
  }

  return parser->Load(device, path_hint, dir_hint);

}

void PlaylistParser::Save(const SongList &songs, const QString &filename, const PlaylistSettingsPage::PathType path_type) const {

  QFileInfo fileinfo(filename);
  QDir dir(fileinfo.path());

  if (!dir.exists()) {
    qLog(Warning) << "Directory does not exist" << dir.path();
    return;
  }

  // Find a parser that supports this file extension
  ParserBase *parser = ParserForExtension(Type::Save, fileinfo.suffix());
  if (!parser) {
    qLog(Warning) << "Unknown filetype" << filename;
    return;
  }

  if (path_type == PlaylistSettingsPage::PathType::Absolute && dir.path() != dir.absolutePath()) {
    dir.setPath(dir.absolutePath());
  }
  else if (path_type != PlaylistSettingsPage::PathType::Absolute && !dir.canonicalPath().isEmpty() && dir.path() != dir.canonicalPath()) {
    dir.setPath(dir.canonicalPath());
  }

  // Open the file
  QFile file(fileinfo.absoluteFilePath());
  if (!file.open(QIODevice::WriteOnly)) {
    qLog(Warning) << "Failed to open" << filename << "for writing.";
    return;
  }

  parser->Save(songs, &file, dir, path_type);

  file.close();

}

bool PlaylistParser::ParserIsSupported(const Type type, ParserBase *parser) const {

  return ((type == Type::Load && parser->load_supported()) || (type == Type::Save && parser->save_supported()));

}

==> strawberry_1.1.1.orig.tar.xz: strawberry-1.1.1/src/playlistparsers/playlistparser.cpp <==
/*
 * Strawberry Music Player
 * This file was part of Clementine.
 * Copyright 2010, David Sansome <me@davidsansome.com>
 *
 * Strawberry is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Strawberry is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Strawberry.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <algorithm>

#include <QObject>
#include <QIODevice>
#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QByteArray>
#include <QString>
#include <QStringList>

#include "core/logging.h"
#include "core/shared_ptr.h"
#include "settings/playlistsettingspage.h"
#include "playlistparser.h"
#include "parserbase.h"
#include "asxiniparser.h"
#include "asxparser.h"
#include "cueparser.h"
#include "m3uparser.h"
#include "plsparser.h"
#include "wplparser.h"
#include "xspfparser.h"

const int PlaylistParser::kMagicSize = 512;

PlaylistParser::PlaylistParser(SharedPtr<CollectionBackendInterface> collection_backend, QObject *parent) : QObject(parent), default_parser_(nullptr) {

  AddParser(new XSPFParser(collection_backend, this));
  AddParser(new M3UParser(collection_backend, this));
  AddParser(new PLSParser(collection_backend, this));
  AddParser(new ASXParser(collection_backend, this));
  AddParser(new AsxIniParser(collection_backend, this));
  AddParser(new CueParser(collection_backend, this));
  AddParser(new WplParser(collection_backend, this));

}

void PlaylistParser::AddParser(ParserBase *parser) {

  if (!default_parser_) {
    default_parser_ = parser;
  }

  parsers_ << parser;
  QObject::connect(parser, &ParserBase::Error, this, &PlaylistParser::Error);

}

QStringList PlaylistParser::file_extensions(const Type type) const {

  QStringList ret;

  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser)) {
      ret << parser->file_extensions();
    }
  }

  std::stable_sort(ret.begin(), ret.end());
  return ret;

}

QStringList PlaylistParser::mime_types(const Type type) const {

  QStringList ret;

  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser) && !parser->mime_type().isEmpty()) {
      ret << parser->mime_type();
    }
  }

  std::stable_sort(ret.begin(), ret.end());

  return ret;

}

QString PlaylistParser::filters(const Type type) const {

  QStringList filters;
  filters.reserve(parsers_.count() + 1);
  QStringList all_extensions;
  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser)) {
      filters << FilterForParser(parser, &all_extensions);
    }
  }

  if (type == Type::Load) {
    filters.prepend(tr("All playlists (%1)").arg(all_extensions.join(QLatin1Char(' '))));
  }

  return filters.join(QLatin1String(";;"));

}

QString PlaylistParser::FilterForParser(const ParserBase *parser, QStringList *all_extensions) {

  const QStringList file_extensions = parser->file_extensions();
  QStringList extensions;
  extensions.reserve(file_extensions.count());
  for (const QString &extension : file_extensions) {
    extensions << QStringLiteral("*.") + extension;
  }

  if (all_extensions) *all_extensions << extensions;

  return tr("%1 playlists (%2)").arg(parser->name(), extensions.join(QLatin1Char(' ')));

}

QString PlaylistParser::default_extension() const {
  QStringList file_extensions = default_parser_->file_extensions();
  return file_extensions[0];
}

QString PlaylistParser::default_filter() const {
  return FilterForParser(default_parser_);
}

ParserBase *PlaylistParser::ParserForExtension(const Type type, const QString &suffix) const {

  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser) && parser->file_extensions().contains(suffix, Qt::CaseInsensitive)) {
      return parser;
    }
  }
  return nullptr;

}

ParserBase *PlaylistParser::ParserForMimeType(const Type type, const QString &mime_type) const {

  for (ParserBase *parser : parsers_) {
    if (ParserIsSupported(type, parser) && !parser->mime_type().isEmpty() && QString::compare(parser->mime_type(), mime_type, Qt::CaseInsensitive) == 0) {
      return parser;
    }
  }
  return nullptr;

}

ParserBase *PlaylistParser::ParserForMagic(const QByteArray &data, const QString &mime_type) const {

  for (ParserBase *parser : parsers_) {
    if ((!mime_type.isEmpty() && mime_type == parser->mime_type()) || parser->TryMagic(data)) {
      return parser;
    }
  }
  return nullptr;

}

SongList PlaylistParser::LoadFromFile(const QString &filename) const {

  QFileInfo fileinfo(filename);

  // Find a parser that supports this file extension
  ParserBase *parser = ParserForExtension(Type::Load, fileinfo.suffix());
  if (!parser) {
    qLog(Error) << "Unknown filetype:" << filename;
    emit Error(tr("Unknown filetype: %1").arg(filename));
    return SongList();
  }

  // Open the file
  QFile file(filename);
  if (!file.open(QIODevice::ReadOnly)) {
    emit Error(tr("Could not open file %1").arg(filename));
    return SongList();
  }

  const SongList songs = parser->Load(&file, filename, fileinfo.absolutePath(), true);
  file.close();

  return songs;

}

SongList PlaylistParser::LoadFromDevice(QIODevice *device, const QString &path_hint, const QDir &dir_hint) const {

  // Find a parser that supports this data
  ParserBase *parser = ParserForMagic(device->peek(kMagicSize));
  if (!parser) {
    return SongList();
  }

  return parser->Load(device, path_hint, dir_hint);

}

void PlaylistParser::Save(const SongList &songs, const QString &filename, const PlaylistSettingsPage::PathType path_type) const {

  QFileInfo fileinfo(filename);
  QDir dir(fileinfo.path());

  if (!dir.exists()) {
    qLog(Error) << "Directory" << dir.path() << "does not exist";
    emit Error(tr("Directory %1 does not exist.").arg(dir.path()));
    return;
  }

  // Find a parser that supports this file extension
  ParserBase *parser = ParserForExtension(Type::Save, fileinfo.suffix());
  if (!parser) {
    qLog(Error) << "Unknown filetype" << filename;
    emit Error(tr("Unknown filetype: %1").arg(filename));
    return;
  }

  if (path_type == PlaylistSettingsPage::PathType::Absolute && dir.path() != dir.absolutePath()) {
    dir.setPath(dir.absolutePath());
  }
  else if (path_type != PlaylistSettingsPage::PathType::Absolute && !dir.canonicalPath().isEmpty() && dir.path() != dir.canonicalPath()) {
    dir.setPath(dir.canonicalPath());
  }

  // Open the file
  QFile file(fileinfo.absoluteFilePath());
  if (!file.open(QIODevice::WriteOnly)) {
    qLog(Error) << "Failed to open" << filename << "for writing.";
    emit Error(tr("Failed to open %1 for writing.").arg(filename));
    return;
  }

  parser->Save(songs, &file, dir, path_type);

  file.close();

}

bool PlaylistParser::ParserIsSupported(const Type type, ParserBase *parser) const {

  return ((type == Type::Load && parser->load_supported()) || (type == Type::Save && parser->save_supported()));

}
